package sun.security.validator;

import java.security.PublicKey;
import java.io.IOException;
import sun.security.x509.NetscapeCertTypeExtension;
import sun.security.util.DerValue;
import sun.security.util.DerInputStream;
import sun.security.x509.X509CertImpl;
import java.util.Set;
import java.security.cert.CertificateException;
import java.security.GeneralSecurityException;
import java.util.Collections;
import sun.security.provider.certpath.AlgorithmChecker;
import java.security.cert.TrustAnchor;
import java.security.cert.CertPathValidatorException;
import sun.security.provider.certpath.UntrustedChecker;
import java.util.Date;
import java.security.AlgorithmConstraints;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Collection;
import java.security.cert.X509Certificate;
import java.util.List;
import javax.security.auth.x500.X500Principal;
import java.util.Map;
import sun.security.util.ObjectIdentifier;

public final class SimpleValidator extends Validator
{
    static final String OID_BASIC_CONSTRAINTS = "2.5.29.19";
    static final String OID_NETSCAPE_CERT_TYPE = "2.16.840.1.113730.1.1";
    static final String OID_KEY_USAGE = "2.5.29.15";
    static final String OID_EXTENDED_KEY_USAGE = "2.5.29.37";
    static final String OID_EKU_ANY_USAGE = "2.5.29.37.0";
    static final ObjectIdentifier OBJID_NETSCAPE_CERT_TYPE;
    private static final String NSCT_SSL_CA = "ssl_ca";
    private static final String NSCT_CODE_SIGNING_CA = "object_signing_ca";
    private final Map<X500Principal, List<X509Certificate>> trustedX500Principals;
    private final Collection<X509Certificate> trustedCerts;
    
    SimpleValidator(String s, Collection<X509Certificate> trustedCerts) {
        super("Simple", s);
        this.trustedCerts = trustedCerts;
        this.trustedX500Principals = new HashMap();
        for (X509Certificate x509Certificate : trustedCerts) {
            X500Principal subjectX500Principal = x509Certificate.getSubjectX500Principal();
            List<X509Certificate> list = (List)this.trustedX500Principals.get(subjectX500Principal);
            if (list == null) {
                list = new ArrayList(2);
                this.trustedX500Principals.put(subjectX500Principal, list);
            }
            list.add(x509Certificate);
        }
    }
    
    @Override
    public Collection<X509Certificate> getTrustedCertificates() {
        return this.trustedCerts;
    }
    
    @Override
    X509Certificate[] engineValidate(X509Certificate[] buildTrustedChain, Collection<X509Certificate> collection, List<byte[]> list, AlgorithmConstraints algorithmConstraints, Object o) throws CertificateException {
        if (buildTrustedChain == null || buildTrustedChain.length == 0) {
            throw new CertificateException("null or zero-length certificate chain");
        }
        buildTrustedChain = this.buildTrustedChain(buildTrustedChain);
        Date validationDate = this.validationDate;
        if (validationDate == null) {
            validationDate = new Date();
        }
        UntrustedChecker o2 = new UntrustedChecker();
        X509Certificate x509Certificate = buildTrustedChain[buildTrustedChain.length - 1];
        try {
            o2.check(x509Certificate);
        }
        catch (CertPathValidatorException ex) {
            throw new ValidatorException("Untrusted certificate: " + x509Certificate.getSubjectX500Principal(), ValidatorException.T_UNTRUSTED_CERT, x509Certificate, ex);
        }
        TrustAnchor anchor = new TrustAnchor(x509Certificate, null);
        AlgorithmChecker o3 = new AlgorithmChecker(anchor, this.variant);
        Object o4 = null;
        if (algorithmConstraints != null) {
            o4 = new AlgorithmChecker(anchor, algorithmConstraints, null, null, this.variant);
        }
        int checkExtensions = buildTrustedChain.length - 1;
        int i = buildTrustedChain.length - 2;
        while (i >= 0) {
            X509Certificate x509Certificate2 = buildTrustedChain[i + 1];
            X509Certificate cert = buildTrustedChain[i];
            try {
                o2.check(cert, Collections.emptySet());
            }
            catch (CertPathValidatorException ex2) {
                throw new ValidatorException("Untrusted certificate: " + cert.getSubjectX500Principal(), ValidatorException.T_UNTRUSTED_CERT, cert, ex2);
            }
            try {
                o3.check(cert, Collections.emptySet());
                if (o4 != null) {
                    o4.check(cert, Collections.emptySet());
                }
            }
            catch (CertPathValidatorException ex3) {
                throw new ValidatorException(ValidatorException.T_ALGORITHM_DISABLED, cert, ex3);
            }
            if (!this.variant.equals("code signing") && !this.variant.equals("jce signing")) {
                cert.checkValidity(validationDate);
            }
            if (!cert.getIssuerX500Principal().equals(x509Certificate2.getSubjectX500Principal())) {
                throw new ValidatorException(ValidatorException.T_NAME_CHAINING, cert);
            }
            try {
                cert.verify(x509Certificate2.getPublicKey());
            }
            catch (GeneralSecurityException ex4) {
                throw new ValidatorException(ValidatorException.T_SIGNATURE_ERROR, cert, ex4);
            }
            if (i != 0) {
                checkExtensions = this.checkExtensions(cert, checkExtensions);
            }
            --i;
        }
        return buildTrustedChain;
    }
    
    private int checkExtensions(X509Certificate x509Certificate, int n) throws CertificateException {
        Set<String> obj = x509Certificate.getCriticalExtensionOIDs();
        if (obj == null) {
            obj = Collections.emptySet();
        }
        int checkBasicConstraints = this.checkBasicConstraints(x509Certificate, obj, n);
        this.checkKeyUsage(x509Certificate, obj);
        this.checkNetscapeCertType(x509Certificate, obj);
        if (!obj.isEmpty()) {
            throw new ValidatorException("Certificate contains unknown critical extensions: " + obj, ValidatorException.T_CA_EXTENSIONS, x509Certificate);
        }
        return checkBasicConstraints;
    }
    
    private void checkNetscapeCertType(X509Certificate x509Certificate, Set<String> set) throws CertificateException {
        if (!this.variant.equals("generic")) {
            if (this.variant.equals("tls client") || this.variant.equals("tls server")) {
                if (!getNetscapeCertTypeBit(x509Certificate, "ssl_ca")) {
                    throw new ValidatorException("Invalid Netscape CertType extension for SSL CA certificate", ValidatorException.T_CA_EXTENSIONS, x509Certificate);
                }
                set.remove("2.16.840.1.113730.1.1");
            }
            else if (this.variant.equals("code signing") || this.variant.equals("jce signing")) {
                if (!getNetscapeCertTypeBit(x509Certificate, "object_signing_ca")) {
                    throw new ValidatorException("Invalid Netscape CertType extension for code signing CA certificate", ValidatorException.T_CA_EXTENSIONS, x509Certificate);
                }
                set.remove("2.16.840.1.113730.1.1");
            }
            else {
                throw new CertificateException("Unknown variant " + this.variant);
            }
        }
    }
    
    static boolean getNetscapeCertTypeBit(X509Certificate x509Certificate, String s) {
        try {
            NetscapeCertTypeExtension netscapeCertTypeExtension;
            if ((x509Certificate instanceof X509CertImpl)) {
                netscapeCertTypeExtension = (NetscapeCertTypeExtension)((X509CertImpl)x509Certificate).getExtension(SimpleValidator.OBJID_NETSCAPE_CERT_TYPE);
                if (netscapeCertTypeExtension == null) {
                    return true;
                }
            }
            else {
                byte[] extensionValue = x509Certificate.getExtensionValue("2.16.840.1.113730.1.1");
                if (extensionValue == null) {
                    return true;
                }
                netscapeCertTypeExtension = new NetscapeCertTypeExtension(new DerValue(new DerInputStream(extensionValue).getOctetString()).getUnalignedBitString().toByteArray());
            }
            return (boolean)netscapeCertTypeExtension.get(s);
        }
        catch (IOException ex) {
            return false;
        }
    }
    
    private int checkBasicConstraints(X509Certificate cert, Set<String> set, int n) throws CertificateException {
        set.remove("2.5.29.19");
        int basicConstraints = cert.getBasicConstraints();
        if (basicConstraints < 0) {
            throw new ValidatorException("End user tried to act as a CA", ValidatorException.T_CA_EXTENSIONS, cert);
        }
        if (!X509CertImpl.isSelfIssued(cert)) {
            if (n <= 0) {
                throw new ValidatorException("Violated path length constraints", ValidatorException.T_CA_EXTENSIONS, cert);
            }
            --n;
        }
        if (n > basicConstraints) {
            n = basicConstraints;
        }
        return n;
    }
    
    private void checkKeyUsage(X509Certificate x509Certificate, Set<String> set) throws CertificateException {
        set.remove("2.5.29.15");
        set.remove("2.5.29.37");
        boolean[] keyUsage = x509Certificate.getKeyUsage();
        if (keyUsage != null && (keyUsage.length < 6 || !keyUsage[5])) {
            throw new ValidatorException("Wrong key usage: expected keyCertSign", ValidatorException.T_CA_EXTENSIONS, x509Certificate);
        }
        return;
    }
    
    private X509Certificate[] buildTrustedChain(X509Certificate[] array) throws CertificateException {
        ArrayList list = new ArrayList(array.length);
        int i = 0;
        while (i < array.length) {
            X509Certificate x509Certificate = array[i];
            X509Certificate trustedCertificate = this.getTrustedCertificate(x509Certificate);
            if (trustedCertificate != null) {
                list.add(trustedCertificate);
                return (X509Certificate[])list.toArray(SimpleValidator.CHAIN0);
            }
            list.add(x509Certificate);
            ++i;
        }
        X509Certificate x509Certificate2 = array[array.length - 1];
        x509Certificate2.getSubjectX500Principal();
        List<Object> list2 = (List)this.trustedX500Principals.get(x509Certificate2.getIssuerX500Principal());
        if (list2 != null) {
            list.add((Object)list2.iterator().next());
            return (X509Certificate[])list.toArray(SimpleValidator.CHAIN0);
        }
        throw new ValidatorException(ValidatorException.T_NO_TRUST_ANCHOR);
    }
    
    private X509Certificate getTrustedCertificate(X509Certificate other) {
        List<Object> list = (List)this.trustedX500Principals.get(other.getSubjectX500Principal());
        if (list == null) {
            return null;
        }
        X500Principal issuerX500Principal = other.getIssuerX500Principal();
        PublicKey publicKey = other.getPublicKey();
        for (X509Certificate x509Certificate : list) {
            if (x509Certificate.equals(other)) {
                return other;
            }
            if (!x509Certificate.getIssuerX500Principal().equals(issuerX500Principal)) {
                continue;
            }
            if (!x509Certificate.getPublicKey().equals(publicKey)) {
                continue;
            }
            return x509Certificate;
        }
        return null;
    }
    
    static {
        OBJID_NETSCAPE_CERT_TYPE = NetscapeCertTypeExtension.NetscapeCertType_Id;
    }
}
